@@ -1,5 +1,5 @@
 from .material import Material
-from .geometry import Geometry
+from .geometry import Geometry as Geometry
 from .solver import Solver
 from .plot import Plot
-from .app import App
+#from .app import App
