@@ -49,7 +49,6 @@ def get_SU(bundle,k):
 
 def get_key(ll,kappa):
 
-
    return (ll,tuple(map(tuple,kappa)))
 
 def unpack(data):
@@ -57,38 +56,55 @@ def unpack(data):
     return data[0],np.array(data[1])
 
 
-def solve_fourier_single(argv):
+def get_kappa_map_from_mat(**argv):
+
+        mat_map = argv['geometry']['elem_mat_map']
+        dim = int(argv['geometry']['meta'][2])
+
+        kappa = argv['material']['kappa']
+
+        kappa = np.array(kappa)
+        if kappa.ndim == 3:
+            return np.array([ list(kappa[i][:dim,:dim])  for i in mat_map])
+        else:
+            return np.array([list(kappa)]*len(argv['geometry']['elems']))
+
+
+def solve_fourier_single(**argv):
    """ Solve Fourier with a single set of kappa """
 
+   mesh = argv['geometry']
 
-   data = None
-
-   if comm.rank == 0:
-
-    mesh = argv['geometry']
-
-    if 'kappa_map_external' in argv.keys(): #read external kappa map
+   if 'kappa_map_external' in argv.keys(): #read external kappa map
      mesh['elem_kappa_map'] = argv['kappa_map_external']
-    else: 
+   else: 
      mesh['elem_kappa_map'] = get_kappa_map_from_mat(**argv)
      #['kappa_map_external'] = mesh['elem_kappa_map']
 
-     
+    
+   kappa = -1 #it means that it will take the map from mesh
 
-    kappa = -1 #it means that it will take the map from mesh
+   F,B,scale = assemble(mesh,argv['material'],kappa)
 
-    F,B,scale = assemble(mesh,kappa)
+   meta,temp,grad =  solve_convergence(argv,kappa,B,splu(F),scale=scale,generation=mesh['generation']+argv.setdefault('additional_heat_source',np.zeros_like(mesh['generation'])))
 
-    meta,temp,grad =  solve_convergence(argv,kappa,B,splu(F),scale=scale)
+   kappa_map = get_kappa_map(mesh,kappa)
 
-    kappa_map = get_kappa_map(mesh,kappa)
+   flux = -np.einsum('cij,cj->ci',kappa_map,grad)*1e9
 
-    flux = -np.einsum('cij,cj->ci',kappa_map,grad)
+   #add an extra dimension if needed
 
-    if argv['verbose']:
+   dim = int(mesh['meta'][2])
+   n_elems = len(mesh['elems'])
+   if dim == 2:flux = np.append(flux, np.zeros((n_elems,1)), axis=1)
+
+   if argv.setdefault('verbose',True):
      fourier_info(meta)
    
-    data = {'meta':meta,'flux':flux,'temperature':temp,'grad':grad}
+   data = {'meta':meta,'flux':flux,'temperature':temp,'grad':grad}
+
+
+   clear_fourier_cache()
 
    return data
 
@@ -96,7 +112,6 @@ def solve_fourier_single(argv):
 
 def get_kappa(mesh,i,j,ll,kappa):
 
-
    if i == j:
     return np.array(kappa[i])
    
@@ -146,10 +161,15 @@ def get_decomposed_directions(mesh,kappa_ll):
     dim = int(mesh['meta'][2])
     normal = mesh['face_normals'][ll,0:dim]
     dist   = mesh['dists'][ll,0:dim]
-
     rot = kappa_map[:dim,:dim]
-    v_orth = np.dot(normal,np.dot(rot,normal))/np.dot(normal,dist)
-    v_non_orth = np.dot(rot,normal) - dist*v_orth
+
+    if ll in (list(mesh['cold_sides']) + list(mesh['hot_sides'])):
+       v_orth = np.dot(normal,np.dot(rot,normal))/np.dot(normal,dist)
+       v_non_orth = np.zeros(dim)
+    else:   
+     v_orth = np.dot(normal,np.dot(rot,normal))/np.dot(normal,dist)
+     v_non_orth = np.dot(rot,normal) - dist*v_orth
+     #print(v_non_orth)
     
     return v_orth,v_non_orth[:dim]
 
@@ -166,90 +186,36 @@ def compute_laplacian(temp,**argv):
 
 
 
-@cached(cache=cache_compute_grad_data,key=lambda mesh,dummy:dummy)
-def compute_grad_data(mesh,TB):
-
-     #Compute deltas-------------------   
-     rr = []
-     for i in mesh['side_per_elem']:
-       rr.append(i*[0])
-
-     #this is wrong
-     for ll in mesh['active_sides'] :
-      kc1,kc2 = mesh['side_elem_map_vec'][ll]
-      ind1    = list(mesh['elem_side_map_vec'][kc1]).index(ll)
-      ind2    = list(mesh['elem_side_map_vec'][kc2]).index(ll)
-
-      delta = 0
-      if ll in mesh['periodic_sides']:
-         delta = mesh['periodic_side_values'][list(mesh['periodic_sides']).index(ll)]
-      else: delta = 0  
-
-      rr[kc1][ind1] = [kc2,kc1, delta] 
-      rr[kc2][ind2] = [kc1,kc2,-delta]
- 
-     return rr
-
-
-def compute_grad(temp,**argv):
-
-   mesh = argv['geometry'] 
-
-   rr = compute_grad_data(mesh,1)
-
-   diff_temp = [[temp[j[0]]-temp[j[1]]+j[2] for j in f] for f in rr]
-   
-   #Add boundary
-   #if 'TB' in argv.keys():
-   # TB  = argv['TB']
-   # for n,(eb,sb) in enumerate(zip(*(mesh['eb'],mesh['sb']))):
-   #   ind               = list(mesh['elem_side_map_vec'][eb]).index(sb)
-   #   diff_temp[eb][ind] = temp[eb]-TB[n]
-
-   gradT = np.array([np.einsum('js,s->j',mesh['weigths'][k,:,:mesh['n_non_boundary_side_per_elem'][k]],np.array(dt)) for k,dt in enumerate(diff_temp)])
-
-
-   return gradT
-
-
-
 
 def compute_secondary_flux(temp,kappa_map,**argv):
 
-   
+   mesh = argv['geometry']
+
    #Compute gradient
-   gradT = compute_grad(temp,**argv)
-   #--------------
+   #gradT = compute_grad(temp,**argv)
+
+   gradT = compute_grad_common(temp,mesh)
 
-   mesh = argv['geometry'] 
    dim = int(mesh['meta'][2])
    n_elems = len(mesh['elems'])
 
-   #-----------SAVE some time--------------------------------------
-   v_non_orth = {}
-   for ll in mesh['active_sides']:
-      (i,j) = mesh['side_elem_map_vec'][ll]
-      if not i==j:
-
-       kappa_loc = get_kappa(mesh,i,j,ll,kappa_map)
-       (v_orth,v_non_orth[ll]) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
-
-       area   = mesh['areas'][ll]   
-       normal = mesh['face_normals'][ll,0:dim]
-       dist   = mesh['dists'][ll,0:dim]
-   #-----------------------------------------------------------------
-
    C = np.zeros(n_elems)
    for ll in mesh['active_sides']:
       area = mesh['areas'][ll] 
       (i,j) = mesh['side_elem_map_vec'][ll]
+      kappa_loc = get_kappa(mesh,i,j,ll,kappa_map)
+      (v_orth,v_non_orth) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
       if not i==j:
        w =  mesh['interp_weigths'][ll]
        F_ave = (w*gradT[i] + (1.0-w)*gradT[j])
-       #tmp = np.dot(F_ave,cache['v_non_orth'][ll])*area
-       tmp = np.dot(F_ave,v_non_orth[ll])*area
+       tmp = np.dot(F_ave,v_non_orth)*area
        C[i] += tmp
        C[j] -= tmp
+      #else:
+      #  if ll in (list(mesh['cold_sides'])+list(mesh['hot_sides'])):
+      #    F_ave = gradT[i] 
+      #    tmp = np.dot(F_ave,v_non_orth)*area
+      #    C[i] += tmp
 
    return C/mesh['volumes'],gradT
 
@@ -265,32 +231,34 @@ def compute_diffusive_thermal_conductivity(temp,kappa_map,**argv):
    k_non_orth = 0
    area_tot = 0
    for ll in mesh['flux_sides']:
-
     (i,j) = mesh['side_elem_map_vec'][ll]
     area = mesh['areas'][ll]
 
-
     kappa_loc = get_kappa(mesh,i,j,ll,kappa_map)
     (v_orth,v_non_orth) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
 
-    deltaT = temp[i] - temp[j] - 1
+    if ll in mesh['hot_sides']: #Isothermal
+     #deltaT = temp[i] - 0.5
+     #deltaT = gradT[i]
+     kappa_eff += np.einsum('ij,j,i->',kappa_loc,argv['grad'][i],mesh['face_normals'][ll][:dim])*area #this is because here flux is in
+    else:
+     deltaT = temp[i] - temp[j] - 1 #Periodic
+     kappa_eff -= v_orth *  deltaT * area
 
-    kappa_eff -= v_orth *  deltaT * area
-
-    if 'grad' in argv.keys():
-     gradT = argv['grad']  
-     w  =  mesh['interp_weigths'][ll]
-     grad_ave = w*gradT[i] + (1.0-w)*gradT[j]
-     k_non_orth += np.dot(grad_ave,v_non_orth)/2 * area
-    area_tot +=area
+    #if 'grad' in argv.keys(): #TODO: add thermalizing boundaries
+    # gradT = argv['grad']  
+     #w  =  mesh['interp_weigths'][ll]
+     #grad_ave = w*gradT[i] + (1.0-w)*gradT[j]
+    # k_non_orth += np.dot(gradT[i],v_non_orth) * area
+    #area_tot +=area
 
    return (kappa_eff+k_non_orth)*mesh['meta'][1]
 
 
-def solve_convergence(argv,kappa,B,SU,log=False,scale=[]):
+def solve_convergence(argv,kappa,B,SU,log=False,scale=[],generation=None):
 
-    argv.setdefault('max_fourier_error',1e-6) 
-    argv.setdefault('max_fourier_iter',10) 
+    argv.setdefault('max_fourier_error',1e-10) 
+    argv.setdefault('max_fourier_iter',20) 
 
     C  = np.zeros_like(B)
 
@@ -302,24 +270,25 @@ def solve_convergence(argv,kappa,B,SU,log=False,scale=[]):
     dim = int(mesh['meta'][2])
     grad = np.zeros((n_elems,dim))
 
-    
-    n_iter = 0;kappa_old = 0;error = 1  
+    n_iter = 0;kappa_old = 0;error = 1;C_old = np.zeros(n_elems)
     while error > argv['max_fourier_error'] and \
                   n_iter < argv['max_fourier_iter'] :
 
-        RHS = B + C
+        RHS = B + C + generation
     
         if len(scale) > 0: RHS *=scale
 
         temp = SU.solve(RHS)
         temp = temp - (max(temp)+min(temp))/2.0
 
+        argv['grad'] = grad
         kappa_eff = compute_diffusive_thermal_conductivity(temp,kappa_map,**argv)
-
-        error = abs((kappa_eff - kappa_old)/kappa_eff)
         kappa_old = kappa_eff
         n_iter +=1
+        #TODO: check error on residual
         C,grad = compute_secondary_flux(temp,kappa_map,**argv)
+        error = (np.linalg.norm(C-C_old))/np.linalg.norm(C)
+        C_old = C.copy()
 
     return [kappa_eff,error,n_iter],temp,grad 
 
@@ -348,11 +317,18 @@ def get_kappa_map(mesh,kappa):
     return kappa
 
 
-@cached(cache=cache_assemble, key=lambda mesh, kappa: hashkey(kappa))
-def assemble(mesh,kappa):
+#@cached(cache=cache_assemble, key=lambda mesh, kappa: hashkey(kappa))
+def assemble(mesh,material,kappa):
 
     kappa_map = get_kappa_map(mesh,kappa)
 
+    #Boundary conductance
+    if 'boundary_conductance' in material.keys():
+        h =  material['boundary_conductance'][0]*1e-9
+    else:
+        h = None
+    #--------------------
+
     n_elems = len(mesh['elems'])
     dim = int(mesh['meta'][2])
 
@@ -365,11 +341,11 @@ def assemble(mesh,kappa):
       (i,j) = mesh['side_elem_map_vec'][ll]
       vi = mesh['volumes'][i]
       vj = mesh['volumes'][j]
+      kappa_loc = get_kappa(mesh,i,j,ll,kappa_map)
       if not i == j:
      
-       kappa_loc = get_kappa(mesh,i,j,ll,kappa_map)
       
-       (v_orth,dummy) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
+       (v_orth,_) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
 
        iff.append(i)
        jff.append(i)
@@ -388,11 +364,33 @@ def assemble(mesh,kappa):
         B[i] += mesh['periodic_side_values'][kk]*v_orth/vi*area
         B[j] -= mesh['periodic_side_values'][kk]*v_orth/vj*area
 
+      if ll in mesh['cold_sides']:    
+          (v_orth,_) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
+          if h == None:  #Dirichlet B.C.
+             a = v_orth 
+          else:  
+             a = v_orth*h/(h + v_orth) 
+          iff.append(i)
+          jff.append(i)
+          dff.append(a/vi*area)
+          B[i] -= 0.5*a/vi*area
+
+      if ll in mesh['hot_sides']:    
+          (v_orth,_) = get_decomposed_directions(mesh,get_key(ll,kappa_loc))
+          if h == None:  #Dirichlet B.C.
+             a = v_orth 
+          else:  
+             a = v_orth*h/(h + v_orth) 
+          iff.append(i)
+          jff.append(i)
+          dff.append(a/vi*area)
+          B[i] += 0.5*a/vi*area
+
+
     F = sp.csc_matrix((np.array(dff),(np.array(iff),np.array(jff))),shape = (n_elems,n_elems))
 
-    #This scale the matrix and fixed zero temperature to a random point
-    scale = fix_instability(F,B)
-    #scale = np.ones(n_elems)
+    scale = fix_instability(F,B,scale=False,floating=(len(mesh['cold_sides'])+len(mesh['hot_sides']))==0)
+
     return F,B,scale
 
 
