@@ -33,10 +33,12 @@ def get_shape(argv):
     argv.setdefault('area_ratio',np.ones(len(argv['base'])))
 
     if argv.setdefault('relative',True):
-     areas =  [argv['porosity']* i /sum(argv['area_ratio'])  for i in argv['area_ratio']]
+     areas = np.array([argv['porosity']* i /sum(argv['area_ratio'])  for i in argv['area_ratio']])
     else: 
-     areas = argv['area_ratio']
-   
+     areas = np.array(argv['area_ratio'])
+
+    areas *=argv['ly']/argv['lx'] #Here we assume that the unit cell reflects that of the whole material
+
     #-------------------
     shapes = []
     n_custom = 0
@@ -47,11 +49,15 @@ def get_shape(argv):
      if shape == 'square':
        shapes.append(np.array(make_polygon(4,area)))
 
+     if shape == 'rectangle':
+       shape_factor = argv.setdefault('shape_factor',1)
+       shapes.append(np.array(make_rectangle(area,shape_factor)))
+
      if shape == 'triangle':
        shapes.append(np.array(make_polygon(3,area)))
 
      elif shape == 'circle':
-       shapes.append(np.array(make_polygon(24,area)))
+       shapes.append(np.array(make_polygon(48,area)))
 
      elif shape == 'custom':
 
